<?php

namespace App\Http\Controllers\admin;

use App\Http\Controllers\Controller;
use App\Models\Booking as ModelsBooking;
use App\Models\Branch;
use App\Models\BranchAsignCity;
use App\Models\CityModel;
use App\Models\CountryModel;
use App\Models\customer;
use App\Models\Service;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Validator;
use Yajra\DataTables\Facades\DataTables;

class Booking extends Controller
{
    public function index()
    {
        $now = date('H');
        $date = date('Y-m-d');
        $time  = date('h:i:sa');
        if ($now == 00) {
            $now = 24;
        }

        if ($now > 20 && $now < 24) {
            $date = date("Y-m-d", strtotime("tomorrow"));
            $time = "09:00:00am";
        }

        $countries = CountryModel::where('status', 1)->get();
        $cities = CityModel::where('status', 1)->get();
        $getservices = Service::where('status', 1)->get();
        return view('admin.booking.index')->with(compact('countries', 'getservices', 'cities', 'date', 'time'));
    }

    public function getParcelPrice($consignor_id = NULL, $service_id = NULL, $consignor_country = NULL, $consignor_city = NULL, $consignee_country = NULL, $consignee_city = NULL)
    {

        $users = $this->getPrice($consignor_id, $service_id, $consignor_country, $consignor_city, $consignee_country, $consignee_city);
        return response()->json(['success' => 'successfull retrieve data', 'data' => $users->toJson()], 200);
    }

    private function getPrice($consignor_id = NULL, $service_id = NULL, $consignor_country = NULL, $consignor_city = NULL, $consignee_country = NULL, $consignee_city = NULL)
    {
        if ($consignor_country == $consignee_country && $consignor_city == $consignee_city) {
            $users = DB::table('price')
                ->where('customer_id', $consignor_id)
                ->where('service_id', $service_id)
                ->where('country_id', $consignee_country)
                ->where('city_id', $consignee_city)
                ->where('price.local', 'Y')
                ->where('price.status', '1')
                ->where('price_type', 'City Wise')->get();
        } else {
            $users = DB::table('price')
                ->where('customer_id', $consignor_id)
                ->where('service_id', $service_id)
                ->where('country_id', $consignee_country)
                ->where('city_id', $consignee_city)
                ->where('price.local', 'N')
                ->where('price.status', '1')
                ->where('price_type', 'City Wise')->get();
        }

        if (Count($users) <= 0) {
            if ($consignor_country == $consignee_country && $consignor_city == $consignee_city) {
                $users = DB::table('price')
                    ->join('zone', 'zone.zone_id', '=', 'price.zone_id')
                    //->where('price.customer_id', $consignor_id)
                    ->where('price.service_id', $service_id)
                    ->where('zone.country_id', $consignee_country)
                    ->where('zone.city_id', $consignee_city)
                    ->where('price.local', 'Y')
                    ->where('price.status', '1')
                    ->where('zone.status', '1')
                    ->where('price.price_type', 'Zone Wise')->get();
            } else {
                $users = DB::table('price')
                    ->join('zone', 'zone.zone_id', '=', 'price.zone_id')
                    // ->where('price.customer_id', $consignor_id)
                    ->where('price.service_id', $service_id)
                    ->where('zone.country_id', $consignee_country)
                    ->where('zone.city_id', $consignee_city)
                    ->where('price.local', 'N')
                    ->where('price.status', '1')
                    ->where('zone.status', '1')
                    ->where('price.price_type', 'Zone Wise')->get();
            }
        }

        if (Count($users) <= 0) {
            if ($consignor_country == $consignee_country && $consignor_city == $consignee_city) {
                $users = DB::table('price')
                    ->where('customer_id', $consignor_id)
                    ->where('service_id', $service_id)
                    ->where('price.local', 'Y')
                    ->where('price.status', '1')
                    ->where('price_type', 'Service Wise')->get();
            } else {
                $users = DB::table('price')
                    ->where('customer_id', $consignor_id)
                    ->where('service_id', $service_id)
                    ->where('price.local', 'N')
                    ->where('price.status', '1')
                    ->where('price_type', 'Service Wise')->get();
            }
        }

        return $users;
    }

    public function addBooking(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'service_id' => 'required|numeric',
                'booking_date' => 'required',
                'booking_time' => 'required',
                'consignee_name' => 'required',
                'consignee_country' => 'required',
                'consignee_city' => 'required',
                'consignee_address' => 'required|regex:/(^[-0-9A-Za-z.,\/ ]+$)/',
                'consignee_contact' => 'required',
                'consignee_post_code' => 'nullable|alpha_num',
                'consignee_email' => 'nullable|email',
                'consignor_id' => 'required|numeric',
                'consignor_name' => 'required|regex:/^[\pL\s\-]+$/u',
                'consignor_country' => 'required|numeric',
                'consignor_city' => 'required|numeric',
                'consignor_address' => 'required|regex:/(^[-0-9A-Za-z.,\/ ]+$)/',
                'consignor_contact' => 'required',
                'consignor_post_code' => 'nullable|alpha_num',
                'consignor_email' => 'required|email',
                'parcel_boking_weight' => 'required|numeric',
                'consignment_details' => 'nullable|regex:/(^[-0-9A-Za-z.,\/ ]+$)/',
                'consignment_note' => 'nullable|regex:/(^[-0-9A-Za-z.,\/ ]+$)/',
                'cod_amount' => 'required_if:service_id,1',
                'shipment_charges' => 'required|numeric',
                'comments' => 'nullable|regex:/(^[-0-9A-Za-z.,\/ ]+$)/',
                'remarks' => 'nullable|regex:/(^[-0-9A-Za-z.,\/ ]+$)/',
                'order_number' => 'nullable|alpha_num',
                'item_code' => 'nullable|alpha_num',
                'slip_no' => 'nullable|alpha_num',
                'quantity' => 'nullable|numeric'
            ]);

            $customMessages = [
                'cod_amount.required_if' => 'The COD amount field is required when service is COD.',
                // Add more custom messages for specific validation rules
            ];

            $validator->setCustomMessages($customMessages);

            if (!$validator->passes()) {
                return response()->json(['error' => $validator->errors()->all()]);
            }

            $booking = new ModelsBooking();
            $booking->id = $request->cn_no;
            $booking->service_id = $request->service_id;
            $booking->customer_type = "corporate";
            $booking->booking_date = $request->booking_date;
            $booking->booking_time = $request->booking_time;
            // $booking->pickup_date = $request->pickup_date;
            $booking->consignee_name = $request->consignee_name;
            $booking->consignee_country = $request->consignee_country;
            $booking->consignee_city = $request->consignee_city;
            $booking->consignee_address = $request->consignee_address;
            $booking->consignee_contact = $request->consignee_contact;
            $booking->consignee_post_code = $request->consignee_post_code;
            $booking->consignee_email = $request->consignee_email;
            $booking->consignor_id = $request->consignor_id;
            $booking->consignor_name = $request->consignor_name;
            $booking->consignor_country = $request->consignor_country;
            $booking->consignor_city = $request->consignor_city;
            $booking->consignor_address = $request->consignor_address;
            $booking->consignor_contact = $request->consignor_contact;
            $booking->consignor_post_code = $request->consignor_post_code;
            $booking->consignor_email = $request->consignor_email;
            $booking->parcel_boking_weight = $request->parcel_boking_weight;
            $booking->parcel_actual_weight = $request->parcel_boking_weight;
            $booking->consignment_details = $request->consignment_details;
            $booking->consignment_note = $request->consignment_note;
            $booking->cod_amount = isset($request->cod_amount) ? $request->cod_amount : 0;
            $booking->shipment_charges = $request->shipment_charges;
            $booking->comments = $request->comments;
            $booking->remarks = $request->remarks;
            $booking->order_number = $request->order_number;
            $booking->item_code = $request->item_code;
            $booking->slip_no = $request->slip_no;
            $booking->quantity = $request->quantity;
            $booking->user_entry = \Auth::User()->id;
            $booking->user_modify = \Auth::User()->id;
            $booking->save();

            $customer = customer::findOrFail($request->consignor_id);
            $customer->current = $booking->id + 1;
            $customer->user_modify = \Auth::User()->id;
            $customer->update();

            return response()->json(['success' => 'data is successfully updated'], 200);
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }


    public function updateBooking(Request $request)
    {
        try {
            if ($request->update == "comments") {
                $validator = Validator::make($request->all(), [
                    'cn_no' => 'required|numeric',
                    'comments' => 'required|regex:/(^[-0-9A-Za-z.,\/ ]+$)/',
                ]);

                if (!$validator->passes()) {
                    return response()->json(['error' => $validator->errors()->all()]);
                }
                $booking = ModelsBooking::findOrFail($request->cn_no);
                $booking->comments = $request->comments;
                $booking->user_modify = \Auth::User()->id;
                $booking->update();
            } else if ($request->update == "weight") {
                $validator = Validator::make($request->all(), [
                    'cn_no' => 'required|numeric',
                    'parcel_actual_weight' => 'required|numeric',
                ]);

                if (!$validator->passes()) {
                    return response()->json(['error' => $validator->errors()->all()]);
                }

                $booking = ModelsBooking::findOrFail($request->cn_no);
                $prices = $this->getPrice($booking->consignor_id, $booking->service_id, $booking->consignor_country, $booking->consignor_city, $booking->consignee_country, $booking->consignee_city);
                $charges = 0;
                $parcel_boking_weight = $request->parcel_actual_weight;
                if ($parcel_boking_weight > 0) {
                    if ($parcel_boking_weight <= 0.5) {
                        $charges = $prices[0]->half_kg;
                    } else if ($parcel_boking_weight > 0.5 && $parcel_boking_weight < 1) {
                        $charges = $prices[0]->kg * ceil($parcel_boking_weight);
                    } else if ($parcel_boking_weight == 1) {
                        $charges = $prices[0]->kg * $parcel_boking_weight;
                    } else if ($parcel_boking_weight > 1 && $parcel_boking_weight < 5) {
                        $parcel_boking_weight = ceil($parcel_boking_weight);
                        $multiply_add = $parcel_boking_weight - 1;
                        $a = $multiply_add * $prices[0]->add_kg;
                        $b = ($prices[0]->kg) * 1;
                        $charges = $a + $b;
                    } else if ($parcel_boking_weight >= 5 && $parcel_boking_weight < 10) {
                        $parcel_boking_weight = ceil($parcel_boking_weight);
                        $multiply_add = $parcel_boking_weight - 1;
                        $a = $multiply_add * ($prices[0]->five_add_kg);
                        $b = ($prices[0]->five_kg) * 1;
                        $charges = $a + $b;
                    } else if ($parcel_boking_weight >= 10 && $parcel_boking_weight < 15) {
                        $parcel_boking_weight = ceil($parcel_boking_weight);
                        $multiply_add = $parcel_boking_weight - 1;
                        $a = $multiply_add * ($prices[0]->ten_add_kg);
                        $b = ($prices[0]->ten_kg) * 1;
                        $charges = $a + $b;
                    } else if ($parcel_boking_weight >= 15 && $parcel_boking_weight < 20) {
                        $parcel_boking_weight = ceil($parcel_boking_weight);
                        $multiply_add = $parcel_boking_weight - 1;
                        $a = $multiply_add * ($prices[0]->fifteen_add_kg);
                        $b = ($prices[0]->fifteen_kg) * 1;
                        $charges = $a + $b;
                    } else if ($parcel_boking_weight >= 20) {
                        $parcel_boking_weight = ceil($parcel_boking_weight);
                        $multiply_add = $parcel_boking_weight - 1;
                        $a = $multiply_add * $prices[0]->twenty_add_kg;
                        $b = $prices[0]->twenty_kg * 1;
                        $charges = $a + $b;
                    }
                }

                $booking->parcel_actual_weight = $request->parcel_actual_weight;
                $booking->shipment_charges = $charges;
                $booking->user_modify = \Auth::User()->id;
                $booking->update();
            }
            return response()->json(['success' => "cn update sucessfully"], 200);
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }

    //dailysheet 

    public function dailysheet()
    {
        $countries = CountryModel::where('status', 1)->get();
        $cities = CityModel::where('status', 1)->get();
        $getservices = Service::where('status', 1)->get();
        $customer = customer::where('status', 1)->get();
        $branches = Branch::where('status', 1)->get();
        $status = DB::table('shipment_status')->get();
        return view('admin.booking.dailysheet')->with(compact('countries', 'getservices', 'cities', 'customer', 'branches', 'status'));
    }

    public function dailysheet_show($cn_no, $city_id,$from_city_id, $customer_id, $payment_mode, $status, $branch_id, $reservation)
    {
        try {
            $booking = DB::table('shipment_booking as sb')
                ->join('customers as cus', 'cus.id', '=', 'sb.consignor_id')
                ->join('city as c_city', 'c_city.id', '=', 'sb.consignor_city')
                ->join('city', 'city.id', '=', 'sb.consignee_city')
                ->join('services as s', 's.id', '=', 'sb.service_id')
                ->join('shipment_status as ss', 'ss.status_id', '=', 'sb.parcel_last_status', 'left');
            if ($cn_no != null && $cn_no != 0)
                $booking->where('sb.id', '=', $cn_no);

            if ($city_id != null && $city_id != 0)
                $booking->whereIn('sb.consignee_city', [$city_id]);


            if ($from_city_id != null && $from_city_id != 0)
                $booking->whereIn('sb.consignor_city', [$from_city_id]);

            if ($customer_id != null && $customer_id != 0)
                $booking->where('sb.consignor_id', '=', $customer_id);

            if ($payment_mode != null && $payment_mode != 0)
                $booking->where('sb.is_cash', '=', $payment_mode);

            if ($status != null && $status != 0)
                $booking->where('sb.parcel_last_status', '=', $status);            

            if ($branch_id != null && $branch_id != 0) {
                $branch_ids = explode(",", branch_filter($branch_id));                
                $booking->whereIn('sb.consignee_city', $branch_ids);
            }


            if ($reservation != null && $reservation != 0) {
                $from_date = "";
                $to_date = "";
                $date_range = $reservation;
                if (!empty($date_range)) {
                    $iparr = explode("|", trim($date_range));
                    $from_date = trim($iparr[0]);
                    $to_date = trim($iparr[1]);
                }

                $booking->whereBetween('sb.booking_date', [$from_date, $to_date]);
            }
            $booking->select('sb.*', 'city.name as destination', 'cus.customer_name', 'c_city.name as consignor_city', 's.service_name', 'cus.return_charges', 'ss.status_name')->get();
            return DataTables::of($booking)
                ->addColumn('action', function ($data) {
                    return table_delete_button($data->id, 'dailysheet');
                })
                ->addColumn('checkbox', function ($data) {
                    return checkbox_show($data->id);
                })
                ->addColumn('parcel_actual_weight', function ($data) {
                    return '<input type="text" cn_no="' . $data->id . '" class="form_control parcel_actual_weight"  name="parcel_actual_weight"  value="' . $data->parcel_actual_weight . '" >';
                })
                ->addColumn('comments', function ($data) {
                    return '<input type="text" cn_no="' . $data->id . '" class="form_control comments"  name="comments"  value="' . $data->comments . '" >';
                })
                ->addColumn('amount', function ($data) {
                    $recivable = 0;
                    $status = $data->parcel_last_status;
                    $status_name = $data->status_name;
                    if ($status_name == "" || $status_name == null)
                        $status_name = "N/A";

                    if ($status == 6 || $status == 5 || $status == 9) {
                        $recivable = $data->cod_amount - $data->shipment_charges - $data->return_charges;
                    } else {
                        $recivable = 0;
                    }
                    $amount = $data->cod_amount - $data->shipment_charges - $recivable;
                    return $amount;
                })
                ->setRowClass(function ($data) {
                    $status = $data->parcel_last_status;
                    return row_color_change($status);
                })
                // ->editColumn('status', function ($data) {
                //     return current_status($data->status);
                // })
                ->addIndexColumn()
                ->rawColumns(['action', 'checkbox', 'status', 'parcel_actual_weight', 'comments','DT_RowIndex'])
                ->make(true);
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }

    //dailysheet close
}
