<?php

namespace App\Http\Controllers\admin;

use App\Http\Controllers\Controller;
use App\Models\Branch;
use Illuminate\Http\Request;
use Yajra\DataTables\Facades\DataTables;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\Rule;

class BranchController extends Controller
{
    public function index()
    {
        return view('admin.branch.index');
    }

    public function create()
    {
        try {
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }
     
    public function store(Request $request)
{
    try {
        $validator = Validator::make($request->all(), [
            'name' => [
                'required',
                'regex:/(^[-0-9A-Za-z.,\/ ]+$)/',
                Rule::unique('branch', 'name'),
            ],
            'contact_no' => 'required|min:11',
            'email' => 'required|email',
            'nic' => 'required|min:13|max:15',
            'per_percel_charges' => 'required|numeric',
            'address' => 'required|regex:/(^[-0-9A-Za-z.,\/ ]+$)/',
        ]);

        if ($validator->fails()) {
            return response()->json(['error' => $validator->errors()->all()]);
        }

        $status = $request->has('status') ? 1 : 0;

        Branch::create([
            'name' => $request->post('name'),
            'contact_no' => $request->post('contact_no'),
            'email' => $request->post('email'),
            'nic' => $request->post('nic'),
            'per_percel_charges' => $request->post('per_percel_charges'),
            'address' => $request->post('address'),
            'status' => $status,
            'user_entry' => auth()->user()->id,
            'user_modify' => auth()->user()->id,
        ]);

        return response()->json(['success' => 'Data is successfully updated'], 200);
    } catch (\Exception $e) {
        dd($e->getMessage());
    }
}

    public function show($id)
    {
        try {
            $branch = Branch::select(['id', 'name','email','contact_no','address','status','per_percel_charges']);
            return DataTables::of($branch)
                ->addColumn('action', function ($data) {
                    return table_edit_delete_button($data->id, 'branch');
                })
                ->addColumn('checkbox', function ($data) {
                    return checkbox_show($data->id);
                })
                ->editColumn('status', function ($data) {
                    return current_status($data->status);
                })        
                ->addIndexColumn()         
                ->rawColumns(['action', 'checkbox', 'status','DT_RowIndex'])
                ->make(true);
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }

    public function edit($id)
    {
        try {
            $branch = Branch::find($id);
            return response()->json(['success' => 'successfull retrieve data', 'data' => $branch->toJson()], 200);
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }

    public function rec_update(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'id' => 'required|numeric',
                'name' => [
                    'required',
                    'regex:/(^[-0-9A-Za-z.,\/ ]+$)/',
                    Rule::unique('branch')->ignore($request->id),
                ],
                'contact_no' => 'required|min:11',
                'email' => 'required|email',
                'nic' => 'required|min:13|max:15',
                'per_percel_charges' => 'required|numeric',
                'address' => 'required|regex:/(^[-0-9A-Za-z.,\/ ]+$)/',
            ]);
    
            if ($validator->fails()) {
                return response()->json(['error' => $validator->errors()->all()]);
            }
    
            $status = $request->has('status') ? 1 : 0;
    

            $branch = Branch::findOrFail($request->id);                                               
            $branch->name = $request->name;           
            $branch->email = $request->email;           
            $branch->nic = $request->nic;
            $branch->contact_no = $request->contact_no;
            $branch->address = $request->address;           
            $branch->per_percel_charges = $request->per_percel_charges;           
            $branch->status = $status;
            $branch->user_modify = \Auth::User()->id;
            $branch->update();
            return response()->json(['success' => 'data is successfully updated'], 200);
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }

    public function destroy($id)
    {
        try {
            $branch = Branch::find($id);
            $branch->delete();
            return response()->json(['success' => 'data is successfully deleted'], 200);
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }

    public function deleteAll(Request $request)
    {
        if ($request->ajax()) {
            $validator = Validator::make($request->all(), [
                'check_all' => 'required'
            ]);
            if (!$validator->passes()) {
                return response()->json(['error' => $validator->errors()->all()]);
            }

            foreach ($request->check_all as $value) {
                Branch::where('id', $value)->delete();
            }
            return response()->json(['success' => 'data is successfully deleted'], 200);
        }
        return 'false';
    }
}