<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\CityModel;
use App\Models\CountryModel;
use Illuminate\Support\Facades\DB;
use Validator;
use Yajra\DataTables\Facades\DataTables as DataTables;
use Illuminate\Validation\Rule;

class CityController extends Controller
{
    public function index()
    {
        $countries = CountryModel::where('status', 1)->latest()->get();  
        return view('admin.city.index')->with(compact('countries'));
    }

    public function create()
    {
        try {
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }
    public function store(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'name' => 'required|unique:city',
                'country_id' => 'required',                
            ]);
            if (!$validator->passes()) {
                return response()->json(['error' => $validator->errors()->all()]);
            }
            $status = 0;
            if($request->post('status') == "on")
            {
                $status = 1;
            }
            CityModel::create([
                'name' => $request->post('name'),
                'country_id' => $request->post('country_id'),
                'status' => $status,
                'user_entry' => \Auth::User()->id,
                'user_modify' => \Auth::User()->id
            ]);
            return response()->json(['success' => 'data is successfully updated'], 200);
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }

    public function show($id)
    {
        try {
            //$city = CityModel::select(['id', 'name','country_id AS country','status']);
            $city = DB::table('city')
                ->join('country', 'country.id', '=', 'city.country_id')                
                ->select('city.*','country.name as country_name')->get();
            return DataTables::of($city)
                ->addColumn('action', function ($data) {
                    return table_edit_delete_button($data->id, 'city');
                })
                ->addColumn('checkbox', function ($data) {
                    return checkbox_show($data->id);
                })
                ->editColumn('status', function ($data) {
                    return current_status($data->status);
                })       
                ->addIndexColumn()        
                ->rawColumns(['action', 'checkbox', 'status','DT_RowIndex'])
                ->make(true);
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }

    public function edit($id)
    {
        try {
            $city = CityModel::find($id);
            return response()->json(['success' => 'successfull retrieve data', 'data' => $city->toJson()], 200);
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }

    public function rec_update(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'city_id' => 'required',
                'name' => [
                    'required',
                    Rule::unique('city')->ignore($request->city_id),
                ],
                'country_id' => 'required',               
            ]);
            if (!$validator->passes()) {
                return response()->json(['error' => $validator->errors()->all()]);
            }
            $status = 0;
            if($request->status == "on")
            {
                $status = 1;
            }
            $city = CityModel::findOrFail($request->city_id);                                               
            $city->name = $request->name;
            $city->country_id = $request->country_id;
            $city->status = $status;            
            $city->user_modify = \Auth::User()->id;
            $city->update();
            return response()->json(['success' => 'data is successfully updated'], 200);
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }

    public function destroy($id)
    {
        try {
            $city = CityModel::find($id);
            $city->delete();
            return response()->json(['success' => 'data is successfully deleted'], 200);
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }

    public function deleteAll(Request $request)
    {
        if ($request->ajax()) {
            $validator = Validator::make($request->all(), [
                'check_all' => 'required'
            ]);
            if (!$validator->passes()) {
                return response()->json(['error' => $validator->errors()->all()]);
            }

            foreach ($request->check_all as $value) {
                CityModel::where('id', $value)->delete();
            }
            return response()->json(['success' => 'data is successfully deleted'], 200);
        }
        return 'false';
    }
}