<?php

namespace App\Http\Controllers\admin;

use App\Http\Controllers\Controller;
use App\Models\Booking;
use App\Models\Branch;
use App\Models\CityModel;
use App\Models\CodBillRecoveryDetail;
use App\Models\CodBillRecoverySummary;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Validator;
use Yajra\DataTables\Facades\DataTables;

class CodBillRecoveryController extends Controller
{
    private $i;
    private $x;
    private $a;
    private $count;
    private $per_parcel_charges;
    private $advance_amount;
    private $branch_id;
    public function index()
    {
        $branches = Branch::where('status', 1)->get();
        $cities = CityModel::where('status', 1)->get();
        return view('admin.codbillrecovery.index_selection')->with(compact('cities', 'branches'));
    }

    public function nextstep()
    {
        return view('admin.codbillrecovery.index');
    }

    public function show($branch_filter = NULL, $destination_filter = NULL, $reservation = NULL)
    {
        try {
            $codbill = DB::table('shipment_booking as sb')
                ->join('customers as cus', 'cus.id', '=', 'sb.consignor_id')
                ->join('city as c_city', 'c_city.id', '=', 'sb.consignor_city')
                ->join('city', 'city.id', '=', 'sb.consignee_city')
                ->join('services as s', 's.id', '=', 'sb.service_id')
                ->join('shipment_status as ss', 'ss.status_id', '=', 'sb.parcel_last_status');

            if ($branch_filter != null && $branch_filter != 0) {
                $branches = Branch::where('status', 1)->where('id', $branch_filter)->get();
                $this->per_parcel_charges = $branches[0]->per_percel_charges;
            } else {
                $this->per_parcel_charges = 0;
            }

            if ($branch_filter != null && $branch_filter != 0) {
                $branch_ids = explode(",", branch_filter($branch_filter));
                $codbill->whereIn('sb.consignee_city', $branch_ids);
            }

            if ($destination_filter != null && $destination_filter != 0)
                $codbill->whereIn('sb.consignee_city', [$destination_filter]);

            $from_date = "";
            $to_date = "";
            $date_range = $reservation;
            if (!empty($date_range)) {
                $iparr = explode("|", trim($date_range));
                $from_date = trim($iparr[0]);
                $to_date = trim($iparr[1]);
            }

            $codbill->whereBetween('sb.pickup_date', [$from_date, $to_date]);
            $codbill->where('sb.parcel_last_status', '!=', '1');
            $codbill->where('sb.is_codbill_rec', '=', '0');
            $codbill->where('sb.is_delete', '=', '0');
            $codbill->whereNotIn('sb.id', DB::table('temp_table')->where('document', '=', 'CODBILL_RECOVERY')->pluck('cn_no'));
            $codbill->orderBy('sb.pickup_date', 'asc');

            $codbill->select(
                'sb.id as cn_no',
                's.id as service_id',
                's.service_name',
                'sb.booking_date',
                'sb.pickup_date',
                'sb.customer_type',
                'sb.consignee_name',
                'sb.consignee_city',
                'sb.parcel_actual_weight',
                'sb.consignor_id',
                'sb.cod_amount',
                'city.name as destination',
                'cus.customer_name',
                'c_city.name as consignor_city',
                'sb.shipment_charges',
                'ss.status_name',
                'cus.return_charges',
                'sb.parcel_last_status'
            )->get();
            return DataTables::of($codbill)
                ->addColumn('checkbox', function ($data) {
                    return checkbox_show($data->cn_no);
                })
                ->addColumn('recevable', function ($data) {
                    $status = $data->parcel_last_status;
                    $recevable = 0;
                    if ($status == 6 || $status == 5 || $status == 9) {
                        $recevable = $data->cod_amount;
                    }
                    return $recevable;
                })
                ->addColumn('remaining', function ($data) {
                    $status = $data->parcel_last_status;
                    $remaining = 0;
                    $recevable = 0;
                    if ($status == 6 || $status == 5 || $status == 9) {
                        $remaining = 0;
                        $recevable = $data->cod_amount;
                    }

                    $remaining = ($data->cod_amount - $this->per_parcel_charges - $recevable);
                    return $remaining;
                })
                ->addColumn('per_parcel_charges', function ($data) {
                    return $this->per_parcel_charges;
                })
                ->addIndexColumn()
                ->rawColumns(['checkbox', 'recevable', 'remaining', 'per_parcel_charges', 'DT_RowIndex'])
                ->make(true);
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }

    public function addBMTempTable(Request $request)
    {
        if ($request->ajax()) {
            $validator = Validator::make($request->all(), [
                'check_all' => 'required',
                'dated' => 'required'
            ]);
            if (!$validator->passes()) {
                return response()->json(['error' => $validator->errors()->all()]);
            }

            $date_range = $request->dated;
            if (!empty($date_range)) {
                $iparr = explode("|", trim($date_range));
                $from_date = trim($iparr[0]);
                $to_date = trim($iparr[1]);
            }

            $branch_id = $request->branch_id;

            foreach ($request->check_all as $value) {
                DB::table('temp_table')->insert([
                    'cn_no' => $value,
                    'document' => "CODBILL_RECOVERY",
                    'branch_id' => $branch_id,
                    'from_date' => $from_date,
                    'to_date' => $to_date,
                    'user_entry' => Auth::User()->id,
                    'user_modify' => Auth::User()->id,
                    'created_at' => getCurrentDateTime(),
                    'updated_at' => getCurrentDateTime()
                ]);
            }
            return response()->json(['Bagmanifest Record Add successfully'], 200);
        }
        return 'false';
    }

    public function showCodRecTempRecord()
    {
        try {
            $this->x = 0;
            $this->i = 0;
            $temp_table = DB::table('temp_table')->where('user_entry', Auth::User()->id)->where('document', "CODBILL_RECOVERY")->get();
            $branches = Branch::where('status', 1)->where('id', $temp_table[0]->branch_id)->get();
            $cod_daily_collection = DB::table('receved_cod_daily_collection')->selectRaw('SUM(amount) AS cod_recovery_collection_amount')->where('branch_id', $branch)->where('status', 0)->get();
            $this->per_parcel_charges = $branches[0]->per_percel_charges;
            $this->advance_amount = $cod_daily_collection[0]->cod_recovery_collection_amount;

            $this->branch_id = $temp_table[0]->branch_id;

            $codbill = DB::table('shipment_booking as sb')
                ->join('customers as cus', 'cus.id', '=', 'sb.consignor_id')
                ->join('city as c_city', 'c_city.id', '=', 'sb.consignor_city')
                ->join('city', 'city.id', '=', 'sb.consignee_city')
                ->join('services as s', 's.id', '=', 'sb.service_id')
                ->join('shipment_status as ss', 'ss.status_id', '=', 'sb.parcel_last_status')
                ->where('sb.parcel_last_status', '!=', '')
                ->where('sb.parcel_last_status', '!=', '1')
                ->where('sb.is_codbill_rec', '=', '0')
                ->where('sb.is_delete', '=', '0')
                ->whereIn('sb.id', DB::table('temp_table')->where('document', '=', 'CODBILL_RECOVERY')->pluck('cn_no'))
                ->orderBy('sb.pickup_date', 'asc')
                ->select(
                    'sb.id as cn_no',
                    's.id as service_id',
                    's.service_name',
                    'sb.booking_date',
                    'sb.pickup_date',
                    'sb.customer_type',
                    'sb.consignee_name',
                    'sb.consignee_city',
                    'sb.parcel_actual_weight',
                    'sb.consignor_id',
                    'sb.cod_amount',
                    'city.name as destination',
                    'cus.customer_name',
                    'c_city.name as consignor_city',
                    'c_city.id as orign',
                    'sb.shipment_charges',
                    'ss.status_name',
                    'cus.return_charges',
                    'sb.parcel_last_status',
                    'cus.fuel_surcharge',
                    'cus.picking_paking'
                )->get();
            $this->count = count($codbill);
            return DataTables::of($codbill)
                ->addColumn('checkbox', function ($data) {
                    $this->a++;
                    return '<input type="checkbox" class="chk_del"  name="chk_del_' . $this->a . '" value="' . $data->cn_no . '" >';
                })
                ->addColumn('recevable', function ($data) {
                    $status = $data->parcel_last_status;
                    $recevable = 0;
                    $return_charges = 0;
                    if ($status == 6 || $status == 5 || $status == 9) {
                        $recevable = $data->cod_amount;
                        $return_charges = $data->cod_amount;
                    }
                    $this->x++;
                    return $recevable . '<input type="hidden" name="totalnum" value="' . $this->count . '" />
                <input type="hidden" name="branch_id_' . $this->x . '"  value="' . $this->branch_id . '" />
                <input type="hidden" name="consignee_name_' . $this->x . '"  value="' . $data->consignee_name . '" />
                <input type="hidden" name="pickup_date_' . $this->x . '"  value="' . $data->pickup_date . '" />
                <input type="hidden" name="consignor_name_' . $this->x . '"  value="' . $data->customer_name . '" />
                <input type="hidden" name="consignor_id_' . $this->x . '"  value="' . $data->consignor_id . '" />
                <input type="hidden" name="weight_' . $this->x . '" id="weight' . $this->x . '" value="' . $data->parcel_actual_weight . '" />
                <input type="hidden" name="orign_' . $this->x . '"  value="' . $data->orign . '" />
                <input type="hidden" name="destination_' . $this->x . '"  value="' . $data->consignee_city . '" />
                <input type="hidden" name="cod_amount_' . $this->x . '" id="cod_amount' . $this->x . '" value="' . $data->cod_amount . '" />
                <input type="hidden" name="shipment_charges_' . $this->x . '" id="shipment_charges' . $this->x . '" value="' . $data->shipment_charges . '" />
                <input type="hidden" name="recivable_' .  $this->x . '" id="recivable' . $this->x . '" value="' . $recevable . '">                
                <input type="hidden" name="status_' . $this->x . '" value=" ' . $status . '" />
                <input type="hidden" name="charges_' . $this->x . '" id="charges' . $this->x . '" value="' . $this->per_parcel_charges . '" />
                <input type="hidden" name="return_charges_' . $this->x . '" id="return_charges' . $this->x . '" value="' . $return_charges . '" />
                <input type="hidden" name="service_type_' . $this->x . '" value="' . $data->service_id . '" />';
                })
                ->addColumn('remaining', function ($data) {
                    $status = $data->parcel_last_status;
                    $remaining = 0;
                    $recevable = 0;
                    if ($status == 6 || $status == 5 || $status == 9) {
                        $remaining = 0;
                        $recevable = $data->cod_amount;
                    }

                    $remaining = ($data->cod_amount - $this->per_parcel_charges - $recevable);

                    $advance_amount = $this->advance_amount;
                    $this->i++;
                    return $remaining . '<input type="hidden" name="amount_' . $this->i . '" id="amount' . $this->i . '" value="' . $remaining . '" />
                    <input type="hidden" name="advance_amount_' . $this->i . '" value="' . $advance_amount . '" />';
                })
                ->addColumn('per_parcel_charges', function ($data) {
                    return $this->per_parcel_charges;
                })
                ->addIndexColumn()
                ->rawColumns(['checkbox', 'recevable', 'remaining', 'per_parcel_charges', 'DT_RowIndex'])
                ->make(true);
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }

    public function addCod(Request $request)
    {
        if ($request->ajax()) {
            $validator = Validator::make($request->all(), [
                'branch_id_1' => 'required|numeric',
                'total_cod_amount' => 'nullable|numeric',
                'total_charges' => 'nullable|numeric',
                'total_return_amound' => 'nullable|numeric',
                'netamount' => 'nullable|numeric',
                'other_amount' => 'nullable|numeric',
                'other_amount_reason' => 'nullable|regex:/(^[-0-9A-Za-z.,\/ ]+$)/',
                'other_recovery' => 'nullable|numeric',
                'other_recovery_reason' => 'nullable|regex:/(^[-0-9A-Za-z.,\/ ]+$)/',
                'advance_amount' => 'nullable|numeric',
                'recived' => 'nullable|numeric',
                'balanceamt' => 'nullable|numeric',
            ]);

            if (!$validator->passes()) {
                return response()->json(['error' => $validator->errors()->all()]);
            }


            DB::beginTransaction();

            try {

                $temp_table = DB::table('temp_table')->where('user_entry', Auth::User()->id)->where('document', "CODBILL_RECOVERY")->get();

                if ($request->totalnum > 0) {
                    $month_name = date('M', strtotime($temp_table[0]->from_date));
                    $month_name = $month_name . date('Y-m-d');
                    $codSummary = new CodBillRecoverySummary;
                    $codSummary->branch_id = $request->branch_id_1;
                    $codSummary->month_name = $month_name;
                    $codSummary->from_date = $temp_table[0]->from_date;
                    $codSummary->to_date = $temp_table[0]->to_date;
                    $codSummary->total_cod_amount = $request->total_cod_amount;
                    $codSummary->total_charges = $request->total_charges;
                    $codSummary->total_return_amound = $request->total_return_amound;
                    $codSummary->other_amount = $request->other_amount;
                    $codSummary->other_amount_reason = $request->other_amount_reason;
                    $codSummary->other_recovery = $request->other_recovery;
                    $codSummary->other_recovery_reason = $request->other_recovery_reason;
                    $codSummary->netamount = $request->netamount;
                    $codSummary->advance_amount = $request->advance_amount;
                    $codSummary->recived = $request->recived;
                    $codSummary->balanceamt = $request->balanceamt;
                    $codSummary->user_entry = Auth::User()->id;
                    $codSummary->user_modify = Auth::User()->id;
                    $codSummary->save();
                }


                $i = 1;
                $exist = "N";
                while ($i <= $request->totalnum) {
                    if (!empty($request->post('chk_del_' . $i))) {
                        $exist = "Y";
                        $codDetail = new CodBillRecoveryDetail;
                        $codDetail->cod_bill_id = $codSummary->id;
                        $codDetail->cn_no = $request->post('chk_del_' . $i);
                        $codDetail->weight = $request->post('weight_' . $i);
                        $codDetail->consignee_name = $request->post('consignee_name_' . $i);
                        $codDetail->orign = $request->post('orign_' . $i);
                        $codDetail->destination = $request->post('destination_' . $i);
                        $codDetail->cod_amount = $request->post('cod_amount_' . $i);
                        $codDetail->charges = $request->post('charges_' . $i);
                        $codDetail->amount = $request->post('amount_' . $i);
                        $codDetail->service_type = $request->post('service_type_' . $i);
                        $codDetail->status = $request->post('status_' . $i);
                        $codDetail->pickup_date = $request->post('pickup_date_' . $i);
                        $codDetail->user_entry = Auth::User()->id;
                        $codDetail->user_modify = Auth::User()->id;
                        $codDetail->save();

                        $booking = Booking::findOrFail($request->post('chk_del_' . $i));
                        $booking->is_codbill_rec  = 1;
                        $booking->user_modify = Auth::User()->id;
                        $booking->update();
                    }
                    $i++;
                }


                if ($exist == "Y") {
                    DB::table('receved_cod_daily_collection')
                        ->where('branch_id', $request->branch_id_1)
                        ->update(['status' => '1']);

                    DB::table('receved_cod_daily_collection')->insert([
                        'date' => date('Y-m-d H:i:s'),
                        'branch_id' => $request->branch_id_1,
                        'bill_number' => $codSummary->id,
                        'bill_amount' => $request->netamount,
                        'recived_recovery_bill' => $request->recived,
                        'advance_amount' => $request->advance_amount,
                        'amount' => $request->balanceamt,
                        'comments' => "Advance Amount",
                        'user_entry' => Auth::User()->id,
                        'user_modify' => Auth::User()->id,
                        'created_at' => getCurrentDateTime(),
                        'updated_at' => getCurrentDateTime()
                    ]);
                }
                DB::table('temp_table')
                    ->where('user_entry', Auth::User()->id)
                    ->where('document', "CODBILL_RECOVERY")->delete();

                DB::commit();
                return response()->json(['success' => 'data is successfully updated'], 200);                
            } catch (\Exception $e) {
                // An error occurred, rollback the transaction
                DB::rollback();
            }
        }
        //return 'false';
    }

    //report section  start
    public function CodBillRecoveryReport()
    {
        $branch = Branch::where('status', 1)->get();
        return view('admin.codbillrecovery.report')->with(compact('branch'));
    }

    public function getBillingMonths($branch_id)
    {
        try {
            $months = CodBillRecoverySummary::select('id as cod_bill_id', 'month_name')->where('branch_id', $branch_id)->get();
            $msg['billing_month'] = $months->toJson();
            return response()->json(['success' => 'successfull retrieve data', $msg], 200);
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }

    public function getBillingMonthDate($branch_id, $month_name)
    {
        try {
            $months = CodBillRecoverySummary::select('from_date', 'to_date')->where('branch_id', $branch_id)->where('id', $month_name)->get();
            $msg['billing_month'] = $months->toJson();
            return response()->json(['success' => 'successfull retrieve data', $msg], 200);
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }

    public function getCodBillReport($branch_id, $month_name)
    {
        try {
            $getreportdetail = DB::table('codbill_recory_detail AS cd')
                ->select('cd.cod_bill_id', 'cd.cn_no', 'cd.pickup_date', 'cd.weight', 'cd.orign', 'cd.destination', 'cd.cod_amount', 'cd.charges', 'cd.amount', 'cd.service_type', 'cd.status', 'c.name AS destination_name', 'ss.status_name', 'services.service_name')
                ->join('codbill_recory_summary AS cs', 'cs.id', '=', 'cd.cod_bill_id')
                ->join('city AS c', 'c.id', '=', 'cd.destination')
                ->join('services', 'services.id', '=', 'cd.service_type')
                ->join('shipment_status AS ss', 'ss.status_id', '=', 'cd.status')
                ->where('cs.branch_id', $branch_id)
                ->where('cd.cod_bill_id', $month_name)
                ->get();

            $html = "";
            $html .= '<table width="100%" class="table table-bordered">
                        <thead>
                            <tr>                                
                                <th>Sr No</th>
                                <th>Date</th>                               
                                <th>CN No</th>
                                <th>Weight</th>
                                <th>Destination</th>
                                <th>COD</th>
                                <th>Charges</th>                                
                                <th>Amount</th>
                                <th>Status</th>
                            </tr>
                        </thead>
                        <tbody>';


            $i = 0;
            if (!empty($getreportdetail)) {
                foreach ($getreportdetail as $row) {
                    $html .= "<tr>
                                <td> " . ($i += 1) . " </td>                                                  
                                <td> " . $row->pickup_date . " </td>
                                <td> " . $row->cn_no . " </td>
                                <td> " . $row->weight . " </td>                                
                                <td> " . $row->destination_name . " </td>
                                <td> " . $row->cod_amount . " </td>
                                <td> " . $row->charges . " </td>                                
                                <td> " . $row->amount . " </td>                                
                                <td> " . $row->status_name . " </td>                                
                            </tr>";
                }
            }


            $html .= '</tbody></table></div>';

            $query_summary = DB::table('codbill_recory_summary as crs')
                ->select(
                    'crs.branch_id',
                    'crs.total_cod_amount',
                    'crs.total_charges',
                    'crs.total_return_amound',
                    'crs.other_amount',
                    'crs.other_amount_reason',
                    'crs.other_recovery',
                    'crs.other_recovery_reason',
                    'crs.netamount',
                    'crs.advance_amount',
                    'crs.recived',
                    'crs.balanceamt'
                )
                ->where('crs.branch_id', $branch_id)
                ->where('crs.id', $month_name)
                ->get();

            if (!empty($query_summary)) {
                $html .= '<div class="row">                
                    <div class="col-md-4">
                        <div class="form-group">
                            <label>Total COD Amount :</label>   
                            <label>' . (isset($query_summary[0]->total_cod_amount) ? $query_summary[0]->total_cod_amount : "") . ' </label>   
                        </div>
                        <!-- /.form group -->
                    </div>                                                       
                    <div class="col-md-4">
                        <div class="form-group">
                            <label>Delivery Charges  :</label>
                            <label>' . (isset($query_summary[0]->total_charges) ? $query_summary[0]->total_charges : "") . '</label>   
                        </div>
                        <!-- /.form group -->
                    </div>
                    <div class="col-md-4">
                        <div class="form-group">
                            <label>Return :</label>
                            <label>' . (isset($query_summary[0]->total_return_amound) ? $query_summary[0]->total_return_amound : "") . '</label>   
                        </div>
                        <!-- /.form group -->
                    </div>                                                      
                </div>
                <div class="row">  
                    <div class="col-md-4">
                        <div class="form-group">
                            <label>Other Charges :</label>
                            <label>' . (isset($query_summary[0]->other_amount) ? $query_summary[0]->other_amount : "") . '</label>
                            <br><label>(' . (isset($query_summary[0]->other_amount_reason) ? $query_summary[0]->other_amount_reason : "") . ')</label>   
                        </div>
                        <!-- /.form group -->
                    </div>
                    <div class="col-md-4">
                        <div class="form-group">
                            <label>Other Recovery :</label>
                            <label>' . (isset($query_summary[0]->other_recovery) ? $query_summary[0]->other_recovery : "") . '</label>
                            <br><label>(' . (isset($query_summary[0]->other_recovery_reason) ? $query_summary[0]->other_recovery_reason : "") .')</label>   
                        </div>
                        <!-- /.form group -->
                    </div>
                    <div class="col-md-4">
                        <div class="form-group">
                            <label>Amount :</label>
                            <label>' . (isset($query_summary[0]->netamount) ? $query_summary[0]->netamount : "") . '</label>   
                        </div>
                        <!-- /.form group -->
                    </div>
                </div>
                <div class="row">  
                    <div class="col-md-4">
                        <div class="form-group">
                            <label>Advance Amount: </label>
                            <label>' . (isset($query_summary[0]->advance_amount) ? $query_summary[0]->advance_amount : "") . '</label>   
                        </div>
                        <!-- /.form group -->
                    </div>
                    <div class="col-md-4">
                        <div class="form-group">
                            <label>Recived Amount: </label>
                            <label>' . (isset($query_summary[0]->recived) ? $query_summary[0]->recived : "") . '</label>   
                        </div>
                        <!-- /.form group -->
                    </div>
                    <div class="col-md-4">
                        <div class="form-group">
                            <label>Balance Amount :</label>
                            <label>' . (isset($query_summary[0]->balanceamt) ? $query_summary[0]->balanceamt : "") . '</label>   
                        </div>
                        <!-- /.form group -->
                    </div>            
                </div>';
            }

            $msg['report_table'] = $html;
            return response()->json(['success' => 'successfull retrieve data', $msg], 200);
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }

    //report close
}
