<?php

namespace App\Http\Controllers\admin;

use App\Http\Controllers\Controller;
use App\Models\Booking;
use App\Models\Charges;
use App\Models\CityModel;
use App\Models\CountryModel;
use App\Models\customer;
use App\Models\Designation;
use App\Models\Service;
use App\Models\Staff;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;
use Yajra\DataTables\Facades\DataTables;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\Rule;
use Illuminate\Support\Facades\Auth;

class CustomerController extends Controller
{
    public function index()
    {
        $staff_ref = Staff::get();
        $designation = Designation::get();
        $countries = CountryModel::where('status', 1)->get();
        $cities = CityModel::where('status', 1)->get();
        $getservices = Service::where('status', 1)->get();
        return view('admin.customer.index')->with(compact('countries', 'cities', 'designation', 'staff_ref', 'getservices'));
    }

    public function create()
    {
        try {
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }
    public function store(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'customer_name' => 'required|regex:/(^[-0-9A-Za-z.,\/ ]+$)/|unique:customers',
                'nic' => 'required|min:13|max:15',
                'contact_no' => 'required|min:11',
                'email' => 'required|email',
                'country_id' => 'required|numeric',
                'city_id' => 'required',
                // 'address' => 'required|regex:/(^[-0-9A-Za-z.,\/ ]+$)/',
                'post_code' => 'required',
                'fuel_surcharge' => 'required',
                'return_charges' => 'required',
                'company' => 'required',
                'ntn' => 'required',
                'reference' => 'alpha',
                'staff_reference_id' => 'numeric',
                'cno_from' => 'numeric',
                'cno_to' => 'numeric',
                'current' => 'numeric',
                'flyer_charges_medium' => 'numeric',
                'flyer_charges_large' => 'numeric',
                'flyer_charges_xlarge' => 'numeric',
                'box_charges_medium' => 'numeric',
                'box_charges_large' => 'numeric',
                'box_charges_xlarge' => 'numeric',
                'picking_paking' => 'numeric',
                'status' => 'required',
                'min_range1' => 'required',
                'min_range2' => 'required',
                'min_range3' => 'required',
                'min_range4' => 'required',
                'max_range1' => 'required',
                'max_range2' => 'required',
                'max_range3' => 'required',
                'max_range4' => 'required',
                'charges1' => 'required|numeric',
                'charges2' => 'required|numeric',
                'charges3' => 'required|numeric',
                'charges4' => 'required|numeric',                
                'password' => 'required',
                'login_status' => 'required|numeric',
            ]);

            if ($validator->fails()) {
                return response()->json(['error' => $validator->errors()->all()]);
            }

            $status = 0;
            if ($request->post('status') == "on") {
                $status = 1;
            }

            try {
                // DB::beginTransaction();
                $customer = customer::create([
                    'customer_name' => $request->post('customer_name'),
                    'nic' => $request->post('nic'),
                    'contact_no' => $request->post('contact_no'),
                    'email' => $request->post('email'),
                    'country_id' => $request->post('country_id'),                    
                    'address' => "",
                    'post_code' => $request->post('post_code'),
                    'fuel_surcharge' => $request->post('fuel_surcharge'),
                    'return_charges' => $request->post('return_charges'),
                    'company' => $request->post('company'),
                    'ntn' => $request->post('ntn'),
                    'reference' => $request->post('reference'),
                    'staff_reference_id' => $request->post('staff_reference_id'),
                    'cno_from' => $request->post('cno_from'),
                    'cno_to' => $request->post('cno_to'),
                    'current' => $request->post('cno_from'),
                    'flyer_charges_medium' => $request->post('flyer_charges_medium'),
                    'flyer_charges_large' => $request->post('flyer_charges_large'),
                    'flyer_charges_xlarge' => $request->post('flyer_charges_xlarge'),
                    'box_charges_medium' => $request->post('box_charges_medium'),
                    'box_charges_large' => $request->post('box_charges_large'),
                    'box_charges_xlarge' => $request->post('box_charges_xlarge'),
                    'picking_paking' => $request->post('picking_paking'),
                    'status' => $status,
                    'user_entry' => Auth::User()->id,
                    'user_modify' => Auth::User()->id
                ]);                                                                             

                if (!empty($request->service)) {
                    foreach ($request->service as $service) {
                        DB::table('customers_asign_service')->insert
                        ([
                            'customer_id' =>$customer->id,
                            'service_id' => $service,
                            'created_at'=> getCurrentDateTime(),
                            'updated_at'=> getCurrentDateTime()                                             
                        ]);
                    }
                }
              
                if (!empty($request->city_id)) {
                    foreach ($request->city_id as $city) {
                        DB::table('customers_asign_city')->insert
                        ([
                            'customer_id' => $customer->id,                        
                            'city_id' => $city,
                            'address' => $request->input('address_'.$city),
                            'created_at'=> getCurrentDateTime(),
                            'updated_at'=> getCurrentDateTime()
                        ]);
                    }
                }

                DB::table('customers_login')->insert
                ([
                    'customer_id' => $customer->id,
                    'email' => $request->post('email'),
                    'password' => Hash::make('password'),
                    'status' => $request->post('login_status'),
                    'user_entry' => Auth::User()->id,
                    'user_modify' => Auth::User()->id,
                    'created_at'=> getCurrentDateTime(),
                    'updated_at'=> getCurrentDateTime()
                ]);


                Charges::create([
                    'charges_type' => "HandlingCharges",
                    'customer_id' => $customer->id,
                    'min_range' => $request->post('min_range1'),
                    'max_range' => $request->post('max_range1'),
                    'charges' => $request->post('charges1'),
                    'medium_qty' => 0,
                    'large_qty' => 0,
                    'xlarge_qty' => 0,
                    'date' => getCurrentDate(),
                    'status' => 0,
                    'user_entry' => Auth::User()->id,
                    'user_modify' => Auth::User()->id
                ]);
                

                Charges::create([
                    'charges_type' => "HandlingCharges",
                    'customer_id' => $customer->id,
                    'min_range' => $request->post('min_range2'),
                    'max_range' => $request->post('max_range2'),
                    'charges' => $request->post('charges2'),
                    'medium_qty' => 0,
                    'large_qty' => 0,
                    'xlarge_qty' => 0,
                    'date' => getCurrentDate(),
                    'status' => 0,
                    'user_entry' => Auth::User()->id,
                    'user_modify' => Auth::User()->id
                ]);


                Charges::create([
                    'charges_type' => "HandlingCharges",
                    'customer_id' => $customer->id,
                    'min_range' => $request->post('min_range3'),
                    'max_range' => $request->post('max_range3'),
                    'charges' => $request->post('charges3'),
                    'medium_qty' => 0,
                    'large_qty' => 0,
                    'xlarge_qty' => 0,
                    'date' => getCurrentDate(),
                    'status' => 0,
                    'user_entry' => Auth::User()->id,
                    'user_modify' => Auth::User()->id
                ]);


                Charges::create([
                    'charges_type' => "HandlingCharges",
                    'customer_id' => $customer->id,
                    'min_range' => $request->post('min_range4'),
                    'max_range' => $request->post('max_range4'),
                    'charges' => $request->post('charges4'),
                    'medium_qty' => 0,
                    'large_qty' => 0,
                    'xlarge_qty' => 0,
                    'date' => getCurrentDate(),
                    'status' => 0,
                    'user_entry' => Auth::User()->id,
                    'user_modify' => Auth::User()->id
                ]);
                return response()->json(['success' => 'data is successfully inserted'.$request->post('status')], 200);
                // DB::commit();
            } catch (\Exception $e) {
                // DB::rollback();                      
                return response()->json(['error' => $e]);
            }
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }

    public function show($id)
    {
        try {            
            $customers = DB::table('customers as c')
                ->join('country', 'country.id', '=', 'c.country_id')                
                ->select('c.*','country.name as country_name')->get();

            return DataTables::of($customers)
                ->addColumn('action', function ($data) {
                    return table_edit_delete_button($data->id, 'customer');
                })
                ->addColumn('checkbox', function ($data) {
                    return checkbox_show($data->id);
                })
                ->editColumn('status', function ($data) {
                    return current_status($data->status);
                })
                ->addColumn('city_id', function ($data) {                    
                    $city_name = "";
                    $cities = DB::table('customers_asign_city as c')
                    ->join('city', 'city.id', '=', 'c.city_id')        
                    ->where('c.customer_id', $data->id)
                    ->select('city.name as city_name')->get();
                    foreach ($cities as $city) {
                        if($city_name == "")
                            $city_name = $city->city_name;
                        else
                        {                           
                            $city_name .= "<br>".$city->city_name;
                        }
                    }
                    return $city_name;
                })
                ->addColumn('address', function ($data) {                    
                    $address = "";
                    $addresses = DB::table('customers_asign_city as c')
                    ->where('c.customer_id', $data->id)
                    ->select('c.address')->get();
                    foreach ($addresses as $add) {
                        if($address == "")
                            $address = $add->address;
                        else
                        {                           
                            $address .= "<br>".$add->address;
                        }                            
                    }
                    return $address;
                })
                ->addIndexColumn()                
                ->rawColumns(['action', 'checkbox', 'status','address','city_id','DT_RowIndex'])
                ->make(true);
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }

    public function edit($id)
    {
        try {
            $customer = customer::find($id);
            $charges = Charges::where('charges_type', "HandlingCharges")->orWhere('customer_id', $id)->get();
            $services = DB::table('customers_asign_service as cas')
                        ->join('services as s','s.id','=','cas.service_id')
                        ->where('customer_id', $id)
                        ->select('s.*')->get();
            
            //$cities = DB::table('customers_asign_city')->where('customer_id', $id)->get();
            $cities = DB::table('customers_asign_city as c')
                    ->join('city', 'city.id', '=', 'c.city_id')        
                    ->where('c.customer_id', $id)
                    ->select('city.id','city.name as city_name','c.address')->get();
            $msg['data'] = $customer->toJson();
            $msg['charges'] = $charges->toJson();
            $msg['result_services_code'] = $services->toJson();
            $msg['cities'] = $cities->toJson();
            return response()->json(['success' => 'successfull retrieve data', $msg], 200);
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }


    public function getCustomerWiseService($customer_id)
    {
        try {                        
            $services = DB::table('customers_asign_service as cas')
                        ->join('services as s','s.id','=','cas.service_id')
                        ->where('customer_id', $customer_id)
                        ->select('s.*')->get();            
            $msg['result_services_code'] = $services->toJson();            
            return response()->json(['success' => 'successfull retrieve data', $msg], 200);
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }

    public function rec_update(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'id' => 'required|numeric',
                'customer_name' => [
                    'required',
                    'regex:/(^[-0-9A-Za-z.,\/ ]+$)/',
                    Rule::unique('customers')->ignore($request->id),
                ],

                'nic' => 'required|min:13|max:15',
                'contact_no' => 'required|min:11',
                'service' => 'required',
                'email' => 'required|email',
                'country_id' => 'required|numeric',
                'city_id' => 'required',
                // 'address' => 'required|regex:/(^[-0-9A-Za-z.,\/ ]+$)/',
                'post_code' => 'required',
                'fuel_surcharge' => 'required',
                'return_charges' => 'required',
                'company' => 'required',
                'ntn' => 'required',
                'reference' => 'alpha',
                'staff_reference_id' => 'numeric',
                'cno_from' => 'numeric',
                'cno_to' => 'numeric',
                'current' => 'numeric',
                'flyer_charges_medium' => 'numeric',
                'flyer_charges_large' => 'numeric',
                'flyer_charges_xlarge' => 'numeric',
                'box_charges_medium' => 'numeric',
                'box_charges_large' => 'numeric',
                'box_charges_xlarge' => 'numeric',
                'picking_paking' => 'numeric',               
                'min_range1' => 'required',
                'min_range2' => 'required',
                'min_range3' => 'required',
                'min_range4' => 'required',
                'max_range1' => 'required',
                'max_range2' => 'required',
                'max_range3' => 'required',
                'max_range4' => 'required',
                'charges1' => 'required|numeric',
                'charges2' => 'required|numeric',
                'charges3' => 'required|numeric',
                'charges4' => 'required|numeric',                
            ]);

            if (!$validator->passes()) {
                return response()->json(['error' => $validator->errors()->all()]);
            }
            $status = 0;
            if ($request->status == "on") {
                $status = 1;
            }

            $customer = customer::findOrFail($request->id);
            $customer->customer_name = $request->customer_name;           
            $customer->nic = $request->nic;           
            $customer->contact_no = $request->contact_no;           
            $customer->email = $request->email;
            $customer->country_id = $request->country_id;           
            $customer->city_id = 0;
            $customer->address = "";           
            $customer->post_code = $request->post_code;           
            $customer->fuel_surcharge = $request->fuel_surcharge;           
            $customer->return_charges = $request->return_charges;           
            $customer->company = $request->company;           
            $customer->ntn = $request->ntn;           
            $customer->reference = $request->reference;           
            $customer->staff_reference_id = $request->staff_reference_id;           
            if($request->cn_chk == "Y")
            {
                $customer->cno_from = $request->cno_from;           
                $customer->cno_to = $request->cno_to;           
                $customer->current = $request->cno_from;
            }
            $customer->flyer_charges_medium = $request->flyer_charges_medium;           
            $customer->flyer_charges_large = $request->flyer_charges_large;           
            $customer->flyer_charges_xlarge = $request->flyer_charges_xlarge;           
            $customer->box_charges_medium = $request->box_charges_medium;           
            $customer->box_charges_large = $request->box_charges_large;           
            $customer->box_charges_xlarge = $request->box_charges_xlarge;           
            $customer->picking_paking = $request->picking_paking;           
            $customer->status = $status;
            $customer->user_modify = Auth::User()->id;
            $customer->update();
            
            DB::table('customers_asign_service')->where('customer_id', $request->id)->delete();
            if (!empty($request->service)) {
                foreach ($request->service as $service) {
                    DB::table('customers_asign_service')->insert
                    ([
                        'customer_id' => $request->id,
                        'service_id' => $service,
                        'created_at'=> getCurrentDateTime(),
                        'updated_at'=> getCurrentDateTime()                                             
                    ]);
                }
            }

            DB::table('customers_asign_city')->where('customer_id', $request->id)->delete();
            if (!empty($request->city_id)) {
                foreach ($request->city_id as $city) {
                    DB::table('customers_asign_city')->insert
                    ([
                        'customer_id' => $request->id,                        
                        'city_id' => $city,
                        'address' => $request->input('address_'.$city),
                        'created_at'=> getCurrentDateTime(),
                        'updated_at'=> getCurrentDateTime()
                    ]);
                }
            }

            Charges::where('customer_id', $request->id)->delete();
            Charges::create([
                'charges_type' => "HandlingCharges",
                'customer_id' => $request->id,
                'min_range' => $request->min_range1,
                'max_range' => $request->max_range1,
                'charges' => $request->charges1,
                'medium_qty' => 0,
                'large_qty' => 0,
                'xlarge_qty' => 0,
                'date' => getCurrentDate(),
                'status' => 0,
                'user_entry' => Auth::User()->id,
                'user_modify' => Auth::User()->id
            ]);
            

            Charges::create([
                'charges_type' => "HandlingCharges",
                'customer_id' => $request->id,
                'min_range' => $request->min_range2,
                'max_range' => $request->max_range2,
                'charges' => $request->charges2,
                'medium_qty' => 0,
                'large_qty' => 0,
                'xlarge_qty' => 0,
                'date' => getCurrentDate(),
                'status' => 0,
                'user_entry' => Auth::User()->id,
                'user_modify' => Auth::User()->id
            ]);


            Charges::create([
                'charges_type' => "HandlingCharges",
                'customer_id' => $request->id,
                'min_range' => $request->min_range3,
                'max_range' => $request->max_range3,
                'charges' => $request->charges3,
                'medium_qty' => 0,
                'large_qty' => 0,
                'xlarge_qty' => 0,
                'date' => getCurrentDate(),
                'status' => 0,
                'user_entry' => Auth::User()->id,
                'user_modify' => Auth::User()->id
            ]);


            Charges::create([
                'charges_type' => "HandlingCharges",
                'customer_id' => $request->id,
                'min_range' => $request->min_range4,
                'max_range' => $request->max_range4,
                'charges' => $request->charges4,
                'medium_qty' => 0,
                'large_qty' => 0,
                'xlarge_qty' => 0,
                'date' => getCurrentDate(),
                'status' => 0,
                'user_entry' => Auth::User()->id,
                'user_modify' => Auth::User()->id
            ]);

            return response()->json(['success' => 'data is successfully updated'], 200);

        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }

    public function destroy($id)
    {
        try {
            $cust = customer::find($id);
            $cust->delete();
            DB::table('customers_asign_service')->where('customer_id', $id)->delete();            
            Charges::where('customer_id', $id)->delete();
            return response()->json(['success' => 'data is successfully deleted'], 200);
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }

    public function deleteAll(Request $request)
    {
        if ($request->ajax()) {
            $validator = Validator::make($request->all(), [
                'check_all' => 'required'
            ]);
            if (!$validator->passes()) {
                return response()->json(['error' => $validator->errors()->all()]);
            }

            foreach ($request->check_all as $value) {
                customer::where('id', $value)->delete();
                DB::table('customers_asign_service')->where('customer_id', $value)->delete();            
                Charges::where('customer_id', $value)->delete();
            }
            return response()->json(['success' => 'data is successfully deleted'], 200);
        }
        return 'false';
    }

    public function customer_login()
    {        
        $customer = customer::where('status', 1)->get();
        return view('admin.customer.customer_login')->with(compact('customer'));
    }

    public function login_show()
    {        
        try {
            $cus_login = DB::table('customers_login as cl')
            ->join('customers as c', 'c.id', '=', 'cl.customer_id')            
            ->select('cl.*','c.customer_name')->get();
            return DataTables::of($cus_login)
                ->addColumn('action', function ($data) {
                    return table_edit_button($data->id, 'customer');
                })
                ->addColumn('checkbox', function ($data) {
                    return checkbox_show($data->id);
                })
                ->editColumn('status', function ($data) {
                    return current_status($data->status);
                })
                ->rawColumns(['action', 'checkbox', 'status'])
                ->make(true);
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }
    
    public function customer_login_update(Request $request)
    {
        try {            
            if ($request->pass_chk == "Y") {
                $validator = Validator::make($request->all(), [
                    'id' => 'required',
                    'customer_id' => 'required',
                    'email' => 'required',
                    'password' => 'required'
                ]);
            }else
            {
                $validator = Validator::make($request->all(), [
                    'id' => 'required',
                    'customer_id' => 'required',
                    'email' => 'required'
                ]);
            }

            
            if (!$validator->passes()) {
                return response()->json(['error' => $validator->errors()->all()]);
            }
            $status = 0;
            if($request->status == "on")
            {
                $status = 1;
            }            
        
            if ($request->pass_chk == "Y") {
                DB::table('customers_login')
                ->where('id', $request->id)
                ->update( [ 
                    'customer_id' => $request->customer_id,
                    'email' => $request->email, 
                    'password' => Hash::make('password'), 
                    'status' => $status,
                    'user_modify' => Auth::User()->id,
                ]);
            }else
            {
                DB::table('customers_login')
                ->where('id', $request->id)
                ->update( [ 
                    'customer_id' => $request->customer_id,
                    'email' => $request->email, 
                    'status' => $status,
                    'user_modify' => Auth::User()->id,
                ]);
            }

            return response()->json(['success' => 'data is successfully updated'], 200);
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }

    public function customer_login_edit($id)
    {       
        try {
            $cus_login = DB::table('customers_login')->where('id', '=', $id)->get();
            return response()->json(['success' => 'successfull retrieve data', 'data' => $cus_login->toJson()], 200);
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }

    public function customer_login_status(Request $request)
    {
        if ($request->ajax()) {
            $validator = Validator::make($request->all(), [
                'check_all' => 'required'
            ]);
            if (!$validator->passes()) {
                return response()->json(['error' => $validator->errors()->all()]);
            }
                        
            foreach ($request->check_all as $value) {                
                 $cus_login = DB::table('customers_login')->where('id', '=', $value)->get();
                $status = 0;
                if($cus_login[0]->status == 0)
                    $status = 1;                

                DB::table('customers_login')
                ->where('id', $value)
                ->update( [
                    'status' => $status,
                    'user_modify' => Auth::User()->id,
                ]);
            }
            return response()->json(['success' => 'Status Updated Successfully'], 200);
        }
        return 'false';
    }

    public function maxCnNo()
    {
        try {
            $max = customer::max('cno_to');
            if ($max == null)
                $max = 1;
            else
                $max++;
            return response()->json(['success' => 'successfull retrieve data', 'data' => $max], 200);
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }
}
