<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Service;
use Validator;
use Yajra\DataTables\Facades\DataTables as DataTables;
use Illuminate\Validation\Rule;

class ServiceController extends Controller
{
    public function index()
    {
        return view('admin.service.index');
    }

    public function create()
    {
        try {
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }
    public function store(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'service_name' => [
                    'required',
                    'regex:/(^[-0-9A-Za-z.,\/ ]+$)/',
                    Rule::unique('services'),
                ],
                'gst' => 'required',     
                'gst' => 'required',                         
            ]);
            if (!$validator->passes()) {
                return response()->json(['error' => $validator->errors()->all()]);
            }
            $status = 0;
            if($request->post('status') == "on")
            {
                $status = 1;
            }

            Service::create([
                'service_name' => $request->post('service_name'),                
                'gst' => $request->post('gst'),                
                'status' => $status,
                'user_entry' => \Auth::User()->id,
                'user_modify' => \Auth::User()->id
            ]);
            return response()->json(['success' => 'data is successfully updated'], 200);
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }

    public function show($id)
    {
        try {
            $servie = Service::select(['id', 'service_name','gst','status']);
            return DataTables::of($servie)
                ->addColumn('action', function ($data) {
                    return table_edit_delete_button($data->id, 'service');
                })
                ->addColumn('checkbox', function ($data) {
                    return checkbox_show($data->id);
                })
                ->editColumn('status', function ($data) {
                    return current_status($data->status);
                })               
                ->addIndexColumn()
                ->rawColumns(['action', 'checkbox', 'status','DT_RowIndex'])
                ->make(true);
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }

    public function edit($id)
    {
        try {
            $service = Service::find($id);
            return response()->json(['success' => 'successfull retrieve data', 'data' => $service->toJson()], 200);
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }

    public function rec_update(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'id' => 'required',
                'service_name' => [
                    'required',
                    'regex:/(^[-0-9A-Za-z.,\/ ]+$)/',
                    Rule::unique('services')->ignore($request->id),
                ],
                'gst' => 'required'                
            ]);
            if (!$validator->passes()) {
                return response()->json(['error' => $validator->errors()->all()]);
            }
            $status = 0;
            if($request->status == "on")
            {
                $status = 1;
            }
            $servie = Service::findOrFail($request->id);                                               
            $servie->service_name = $request->service_name;           
            $servie->gst = $request->gst;                     
            $servie->status = $status;
            $servie->user_modify = \Auth::User()->id;
            $servie->update();
            return response()->json(['success' => 'data is successfully updated'], 200);
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }

    public function destroy($id)
    {
        try {
            $servie = Service::find($id);
            $servie->delete();
            return response()->json(['success' => 'data is successfully deleted'], 200);
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }

    public function deleteAll(Request $request)
    {
        if ($request->ajax()) {
            $validator = Validator::make($request->all(), [
                'check_all' => 'required'
            ]);
            if (!$validator->passes()) {
                return response()->json(['error' => $validator->errors()->all()]);
            }

            foreach ($request->check_all as $value) {
                Service::where('id', $value)->delete();
            }
            return response()->json(['success' => 'data is successfully deleted'], 200);
        }
        return 'false';
    }    
}
