<?php

namespace App\Http\Controllers\admin;

use App\Http\Controllers\Controller;
use App\Models\CityModel;
use App\Models\CountryModel;
use App\Models\customer;
use App\Models\VehicleWisePrice;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Yajra\DataTables\Facades\DataTables;
use Validator;

class VehicleWPriceController extends Controller
{
    public function index()
    {
        $customer = customer::where('status', 1)->get();
        $cities = CityModel::where('status', 1)->get();
        $countries = CountryModel::where('status', 1)->get();
        return view('admin.vehicle_wise_price.index')->with(compact('customer', 'cities', 'countries'));
    }

    public function create()
    {
        try {
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }
    public function store(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'price_type' => 'required',
                'customer_id' => 'required|numeric',
                'from_city_id' => 'required|numeric',
                'to_city_id' => 'required|numeric',
                'fixed_amount' => 'required|numeric',
                'per_kelometer_amount' => 'required|numeric',
                'booking_amount' => 'required|numeric'
            ]);


            if (!$validator->passes()) {
                return response()->json(['error' => $validator->errors()->all()]);
            }
            $status = 0;
            if ($request->post('status') == "on") {
                $status = 1;
            }
           
            $price = new VehicleWisePrice;
            $price->price_type = $request->price_type;
            $price->customer_id = $request->customer_id;
            $price->from_city_id = $request->from_city_id;
            $price->to_city_id = $request->to_city_id;
            $price->fixed_amount = $request->fixed_amount;
            $price->per_kelometer_amount = $request->per_kelometer_amount;
            $price->booking_amount = $request->booking_amount;
            $price->status = $status;
            $price->user_entry = \Auth::User()->id;
            $price->user_modify = \Auth::User()->id;
            $price->save();
            return response()->json(['success' => 'data is successfully updated'], 200);
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }

    public function show($id)
    {
        try {
            $route = DB::table('vehicle_wise_price as wp')
                ->join('customers as cus', 'cus.id', '=', 'wp.customer_id')
                ->join('city as fc', 'fc.id', '=', 'wp.from_city_id')
                ->join('city as tc', 'tc.id', '=', 'wp.to_city_id')
                ->select('wp.*', 'cus.customer_name', 'fc.name as city_from_name', 'tc.name as city_to_name')->get();
            return DataTables::of($route)
                ->addColumn('action', function ($data) {
                    return table_edit_delete_button($data->id, 'vehicle_wise_price');
                })
                ->addColumn('checkbox', function ($data) {
                    return checkbox_show($data->id);
                })
                ->editColumn('status', function ($data) {
                    return current_status($data->status);
                })
                ->addIndexColumn()
                ->rawColumns(['action', 'checkbox', 'status','DT_RowIndex'])
                ->make(true);
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }

    public function edit($id)
    {
        try {
            $price = VehicleWisePrice::find($id);
            return response()->json(['success' => 'successfull retrieve data', 'data' => $price->toJson()], 200);
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }

    public function rec_update(Request $request)
    {
        try {

            $validator = Validator::make($request->all(), [
                'id' => 'required',
                'price_type' => 'required',
                'customer_id' => 'required|numeric',
                'from_city_id' => 'required|numeric',
                'to_city_id' => 'required|numeric',
                'fixed_amount' => 'required|numeric',
                'per_kelometer_amount' => 'required|numeric',
                'booking_amount' => 'required|numeric'
            ]);

            if (!$validator->passes()) {
                return response()->json(['error' => $validator->errors()->all()]);
            }

            $status = 0;
            if ($request->post('status') == "on") {
                $status = 1;
            }
           
            $price = VehicleWisePrice::findOrFail($request->id);
            $price->price_type = $request->price_type;
            $price->customer_id = $request->customer_id;
            $price->from_city_id = $request->from_city_id;
            $price->to_city_id = $request->to_city_id;
            $price->fixed_amount = $request->fixed_amount;
            $price->per_kelometer_amount = $request->per_kelometer_amount;
            $price->booking_amount = $request->booking_amount;            
            $price->status = $status;
            $price->user_modify = \Auth::User()->id;
            $price->update();
            return response()->json(['success' => 'data is successfully updated'], 200);
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }

    public function destroy($id)
    {
        try {
            $price = VehicleWisePrice::find($id);
            $price->delete();
            return response()->json(['success' => 'data is successfully deleted'], 200);
        } catch (\Exception $e) {
            dd($e->getMessage());
        }
    }

    public function deleteAll(Request $request)
    {
        if ($request->ajax()) {
            $validator = Validator::make($request->all(), [
                'check_all' => 'required'
            ]);
            if (!$validator->passes()) {
                return response()->json(['error' => $validator->errors()->all()]);
            }

            foreach ($request->check_all as $value) {
                VehicleWisePrice::where('id', $value)->delete();
            }
            return response()->json(['success' => 'data is successfully deleted'], 200);
        }
        return 'false';
    }
}
